@@ -51,10 +51,10 @@ import Annex.InodeSentinal
 import Upgrade
 import Annex.Tmp
 import Utility.UserInfo
-import qualified Utility.RawFilePath as R
-import Utility.ThreadScheduler
 import Annex.Perms
 #ifndef mingw32_HOST_OS
+import Utility.ThreadScheduler
+import qualified Utility.RawFilePath as R
 import Utility.FileMode
 import System.Posix.User
 import qualified Utility.LockFile.Posix as Posix
@@ -62,8 +62,8 @@ import qualified Utility.LockFile.Posix as Posix
 
 import qualified Data.Map as M
 import Control.Monad.IO.Class (MonadIO)
-import System.PosixCompat.Files (ownerReadMode, isNamedPipe)
 #ifndef mingw32_HOST_OS
+import System.PosixCompat.Files (ownerReadMode, isNamedPipe)
 import Data.Either
 import qualified System.FilePath.ByteString as P
 import Control.Concurrent.Async
@@ -79,7 +79,7 @@ checkInitializeAllowed a = guardSafeToUseRepo $ noAnnexFileContent' >>= \case
 	Just noannexmsg -> do
 		warning "Initialization prevented by .noannex file (remove the file to override)"
 		unless (null noannexmsg) $
-			warning noannexmsg
+			warning (UnquotedString noannexmsg)
 		giveup "Not initialized."
 
 initializeAllowed :: Annex Bool
@@ -118,6 +118,9 @@ initialize mdescription mversion = checkInitializeAllowed $ \initallowed -> do
 	initSharedClone sharedclone
 	
 	u <- getUUID
+	when (u == NoUUID) $
+		giveup "Failed to read annex.uuid from git config after setting it. This should never happen. Please file a bug report."
+	
 	{- Avoid overwriting existing description with a default
 	 - description. -}
 	whenM (pure (isJust mdescription) <||> not . M.member u <$> uuidDescMapRaw) $
@@ -272,7 +275,7 @@ probeCrippledFileSystem = withEventuallyCleanedOtherTmp $ \tmp -> do
 		(Just (freezeContent' UnShared))
 		(Just (thawContent' UnShared))
 		=<< hasFreezeHook
-	mapM_ warning warnings
+	mapM_ (warning . UnquotedString) warnings
 	return r
 
 probeCrippledFileSystem'
@@ -404,8 +407,8 @@ checkSqliteWorks = do
 		Right () -> return ()
 		Left e -> do
 			showLongNote $ "Detected a filesystem where Sqlite does not work."
-			showLongNote $ "(" ++ show e ++ ")"
-			showLongNote $ "To work around this problem, you can set annex.dbdir " ++
+			showLongNote $ UnquotedString $ "(" ++ show e ++ ")"
+			showLongNote $ "To work around this problem, you can set annex.dbdir " <>
 				"to a directory on another filesystem."
 			showLongNote $ "For example: git config annex.dbdir $HOME/cache/git-annex"
 			giveup "Not initialized."
