@@ -5,7 +5,7 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
-{-# LANGUAGE ScopedTypeVariables, DeriveDataTypeable #-}
+{-# LANGUAGE ScopedTypeVariables, DeriveDataTypeable, OverloadedStrings #-}
 
 module Annex.NumCopies (
 	module Types.NumCopies,
@@ -57,11 +57,11 @@ deprecatedNumCopies = annexNumCopies <$> Annex.getGitConfig
 
 {- Value forced on the command line by --numcopies. -}
 getForcedNumCopies :: Annex (Maybe NumCopies)
-getForcedNumCopies = Annex.getState Annex.forcenumcopies
+getForcedNumCopies = Annex.getRead Annex.forcenumcopies
 
 {- Value forced on the command line by --mincopies. -}
 getForcedMinCopies :: Annex (Maybe MinCopies)
-getForcedMinCopies = Annex.getState Annex.forcemincopies
+getForcedMinCopies = Annex.getRead Annex.forcemincopies
 
 {- NumCopies value from any of the non-.gitattributes configuration
  - sources. -}
@@ -185,7 +185,7 @@ getNumMinCopiesAttr file =
 
 {- Checks if numcopies are satisfied for a file by running a comparison
  - between the number of (not untrusted) copies that are
- - belived to exist, and the configured value.
+ - believed to exist, and the configured value.
  -
  - This is good enough for everything except dropping the file, which
  - requires active verification of the copies.
@@ -197,13 +197,17 @@ numCopiesCheck file key vs = do
 
 numCopiesCheck' :: RawFilePath -> (Int -> Int -> v) -> [UUID] -> Annex v
 numCopiesCheck' file vs have = do
-	needed <- fst <$> getFileNumMinCopies file
-	return $ length have `vs` fromNumCopies needed
+	needed <- fromNumCopies . fst <$> getFileNumMinCopies file
+	let nhave = length have
+	explain (ActionItemTreeFile file) $ Just $ UnquotedString $
+		"has " ++ show nhave ++ " " ++ pluralCopies nhave ++ 
+		", and the configured annex.numcopies is " ++ show needed
+	return $ nhave `vs` needed
 
 data UnVerifiedCopy = UnVerifiedRemote Remote | UnVerifiedHere
 	deriving (Ord, Eq)
 
-{- Verifies that enough copies of a key exist amoung the listed remotes,
+{- Verifies that enough copies of a key exist among the listed remotes,
  - to safely drop it, running an action with a proof if so, and
  - printing an informative message if not.
  -}
@@ -277,24 +281,25 @@ notEnoughCopies :: Key -> NumCopies -> MinCopies -> [VerifiedCopy] -> [UUID] ->
 notEnoughCopies key neednum needmin have skip bad nolocmsg lockunsupported = do
 	showNote "unsafe"
 	if length have < fromNumCopies neednum
-		then showLongNote $
+		then showLongNote $ UnquotedString $
 			"Could only verify the existence of " ++
 			show (length have) ++ " out of " ++ show (fromNumCopies neednum) ++ 
-			" necessary " ++ pluralcopies (fromNumCopies neednum)
+			" necessary " ++ pluralCopies (fromNumCopies neednum)
 		else do
-			showLongNote $ "Unable to lock down " ++ show (fromMinCopies needmin) ++ 
-				" " ++ pluralcopies (fromMinCopies needmin) ++ 
+			showLongNote $ UnquotedString $ "Unable to lock down " ++ show (fromMinCopies needmin) ++ 
+				" " ++ pluralCopies (fromMinCopies needmin) ++ 
 				" of file necessary to safely drop it."
 			if null lockunsupported
 				then showLongNote "(This could have happened because of a concurrent drop, or because a remote has too old a version of git-annex-shell installed.)"
-				else showLongNote $ "These remotes do not support locking: "
+				else showLongNote $ UnquotedString $ "These remotes do not support locking: "
 					++ Remote.listRemoteNames lockunsupported
 
 	Remote.showTriedRemotes bad
 	Remote.showLocations True key (map toUUID have++skip) nolocmsg
-  where
-	pluralcopies 1 = "copy"
-	pluralcopies _ = "copies"
+
+pluralCopies :: Int -> String
+pluralCopies 1 = "copy"
+pluralCopies _ = "copies"
 
 {- Finds locations of a key that can be used to get VerifiedCopies,
  - in order to allow dropping the key.
