@@ -5,6 +5,8 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
+{-# LANGUAGE OverloadedStrings #-}
+
 module Command.FromKey where
 
 import Command
@@ -18,11 +20,12 @@ import Annex.Link
 import Annex.FileMatcher
 import Annex.Ingest
 import Git.FilePath
+import Utility.Url
 
 import Network.URI
 
 cmd :: Command
-cmd = notBareRepo $ withGlobalOptions [jsonOptions] $
+cmd = notBareRepo $ withAnnexOptions [jsonOptions] $
 	command "fromkey" SectionPlumbing "adds a file using a specific key"
 		(paramRepeating (paramPair paramKey paramPath))
 		(seek <$$> optParser)
@@ -46,7 +49,7 @@ seek o = do
 		-- older way of enabling batch input, does not support BatchNull
 		(NoBatch, []) -> seekBatch matcher (BatchFormat BatchLine (BatchKeys False))
 		(NoBatch, ps) -> do
-			force <- Annex.getState Annex.force
+			force <- Annex.getRead Annex.force
 			withPairs (commandAction . start matcher force) ps
 
 seekBatch :: AddUnlockedMatcher -> BatchFormat -> CommandSeek
@@ -89,7 +92,7 @@ keyOpt :: String -> Key
 keyOpt = either giveup id . keyOpt'
 
 keyOpt' :: String -> Either String Key
-keyOpt' s = case parseURI s of
+keyOpt' s = case parseURIPortable s of
 	Just u | not (isKeyPrefix (uriScheme u)) ->
 		Right $ Backend.URL.fromUrl s Nothing
 	_ -> case deserializeKey s of
@@ -129,7 +132,7 @@ perform matcher key file = lookupKeyNotHidden file >>= \case
 		| otherwise -> hasothercontent
   where
 	hasothercontent = do
-		warning $ fromRawFilePath file ++ " already exists with different content"
+		warning $ QuotedPath file <> " already exists with different content"
 		next $ return False
 	
 	linkunlocked = linkFromAnnex key file Nothing >>= \case
