@@ -16,11 +16,13 @@ import qualified Remote
 import qualified Types.Backend
 import qualified Backend
 import Annex.Content
+#ifndef mingw32_HOST_OS
+import Annex.Version
 import Annex.Content.Presence
+#endif
 import Annex.Content.Presence.LowLevel
 import Annex.Perms
 import Annex.Link
-import Annex.Version
 import Logs.Location
 import Logs.Trust
 import Logs.Activity
@@ -40,7 +42,6 @@ import qualified Database.Keys
 import qualified Database.Fsck as FsckDb
 import Types.CleanupActions
 import Types.Key
-import Types.ActionItem
 import qualified Utility.RawFilePath as R
 
 import Data.Time.Clock.POSIX
@@ -72,7 +73,7 @@ data IncrementalOpt
 optParser :: CmdParamsDesc -> Parser FsckOptions
 optParser desc = FsckOptions
 	<$> cmdParams desc
-	<*> optional (parseRemoteOption <$> strOption 
+	<*> optional (mkParseRemoteOption <$> strOption 
 		( long "from" <> short 'f' <> metavar paramRemote 
 		<> help "check remote"
 		<> completeRemotes
@@ -112,7 +113,7 @@ seek o = startConcurrency commandStages $ do
 	cleanupIncremental i
 	void $ tryIO $ recordActivity Fsck u
   where
-	ww = WarnUnmatchLsFiles
+	ww = WarnUnmatchLsFiles "fsck"
 
 checkDeadRepo :: UUID -> Annex ()
 checkDeadRepo u =
@@ -158,14 +159,15 @@ performRemote key afile backend numcopies remote =
 	dispatch =<< Remote.hasKey remote key
   where
 	dispatch (Left err) = do
-		showNote err
+		showNote (UnquotedString err)
 		return False
 	dispatch (Right True) = withtmp $ \tmpfile ->
 		getfile tmpfile >>= \case
 			Nothing -> go True Nothing
 			Just (Right verification) -> go True (Just (tmpfile, verification))
 			Just (Left _) -> do
-				warning (decodeBS (actionItemDesc ai) ++ ": failed to download file from remote")
+				warning $ actionItemDesc ai
+					<> ": failed to download file from remote"
 				void $ go True Nothing
 				return False
 	dispatch (Right False) = go False Nothing
@@ -257,12 +259,13 @@ fixLink key file = do
  - to the other location.
  -}
 fixObjectLocation :: Key -> Annex Bool
-fixObjectLocation key = do
 #ifdef mingw32_HOST_OS
+fixObjectLocation _key = do
 	-- Windows does not allow locked files to be renamed, but annex
 	-- links are also not used on Windows.
 	return True
 #else
+fixObjectLocation key = do
 	loc <- calcRepo (gitAnnexLocation key)
 	idealloc <- calcRepo (gitAnnexLocation' (const (pure True)) key)
 	if loc == idealloc
@@ -317,7 +320,7 @@ verifyLocationLog key keystatus ai = do
 			KeyLockedThin -> thawContent obj
 			_ -> freezeContent obj
 		checkContentWritePerm obj >>= \case
-			Nothing -> warning $ "** Unable to set correct write mode for " ++ fromRawFilePath obj ++ " ; perhaps you don't own that file, or perhaps it has an xattr or ACL set"
+			Nothing -> warning $ "** Unable to set correct write mode for " <> QuotedPath obj <> " ; perhaps you don't own that file, or perhaps it has an xattr or ACL set"
 			_ -> return ()
 	whenM (liftIO $ R.doesPathExist $ parentDir obj) $
 		freezeContentDir obj
@@ -328,7 +331,7 @@ verifyLocationLog key keystatus ai = do
 	 - config was set. -}
 	whenM (pure present <&&> (not <$> Backend.isCryptographicallySecure key)) $
 		whenM (annexSecureHashesOnly <$> Annex.getGitConfig) $
-			warning $ "** Despite annex.securehashesonly being set, " ++ fromRawFilePath obj ++ " has content present in the annex using an insecure " ++ decodeBS (formatKeyVariety (fromKey keyVariety key)) ++ " key"
+			warning $ "** Despite annex.securehashesonly being set, " <> QuotedPath obj <> " has content present in the annex using an insecure " <> UnquotedString (decodeBS (formatKeyVariety (fromKey keyVariety key))) <> " key"
 
 	verifyLocationLog' key ai present u (logChange key u)
 
@@ -348,9 +351,9 @@ verifyLocationLog' key ai present u updatestatus = do
 		(False, True) -> do
 			fix InfoMissing
 			warning $
-				"** Based on the location log, " ++
-				decodeBS (actionItemDesc ai) ++
-				"\n** was expected to be present, " ++
+				"** Based on the location log, " <>
+				actionItemDesc ai <>
+				"\n** was expected to be present, " <>
 				"but its content is missing."
 			return False
 		(False, False) -> do
@@ -388,10 +391,10 @@ verifyRequiredContent key ai@(ActionItemAssociatedFile afile _) = case afile of
 			else do
 				missingrequired <- Remote.prettyPrintUUIDs "missingrequired" missinglocs
 				warning $
-					"** Required content " ++
-					decodeBS (actionItemDesc ai) ++
-					" is missing from these repositories:\n" ++
-					missingrequired
+					"** Required content " <>
+					actionItemDesc ai <>
+					" is missing from these repositories:\n" <>
+					UnquotedString missingrequired
 				return False
 verifyRequiredContent _ _ = return True
 
@@ -462,13 +465,11 @@ checkKeySizeOr bad key file ai = case fromKey keySize key of
 		return same
 	badsize a b = do
 		msg <- bad key
-		warning $ concat
-			[ decodeBS (actionItemDesc ai)
-			, ": Bad file size ("
-			, compareSizes storageUnits True a b
-			, "); "
-			, msg
-			]
+		warning $ actionItemDesc ai
+			<> ": Bad file size ("
+			<> UnquotedString (compareSizes storageUnits True a b)
+			<> "); "
+			<> UnquotedString msg
 
 {- Check for keys that are upgradable.
  -
@@ -480,13 +481,12 @@ checkKeyUpgrade :: Backend -> Key -> ActionItem -> AssociatedFile -> Annex Bool
 checkKeyUpgrade backend key ai (AssociatedFile (Just file)) =
 	case Types.Backend.canUpgradeKey backend of
 		Just a | a key -> do
-			warning $ concat
-				[ decodeBS (actionItemDesc ai)
-				, ": Can be upgraded to an improved key format. "
-				, "You can do so by running: git annex migrate --backend="
-				, decodeBS (formatKeyVariety (fromKey keyVariety key)) ++ " "
-				, decodeBS file
-				]
+			warning $ actionItemDesc ai
+				<> ": Can be upgraded to an improved key format. "
+				<> "You can do so by running: git annex migrate --backend="
+				<> UnquotedByteString (formatKeyVariety (fromKey keyVariety key))
+				<> " "
+				<> QuotedPath file
 			return True
 		_ -> return True
 checkKeyUpgrade _ _ _ (AssociatedFile Nothing) =
@@ -531,11 +531,9 @@ checkBackendOr bad backend key file ai =
 			ok <- verifier key file
 			unless ok $ do
 				msg <- bad key
-				warning $ concat
-					[ decodeBS (actionItemDesc ai)
-					, ": Bad file content; "
-					, msg
-					]
+				warning $ actionItemDesc ai
+					<> ": Bad file content; "
+					<> UnquotedString msg
 			return ok
 		Nothing -> return True
 
@@ -559,17 +557,15 @@ checkInodeCache key content mic ai = case mic of
 				withTSDelta (liftIO . genInodeCache content) >>= \case
 					Nothing -> noop
 					Just ic' -> whenM (compareInodeCaches ic ic') $ do
-						warning $ concat
-							[ decodeBS (actionItemDesc ai)
-							, ": Stale or missing inode cache; updating."
-							]
+						warning $ actionItemDesc ai
+							<> ": Stale or missing inode cache; updating."
 						Database.Keys.addInodeCaches key [ic]
 
 checkKeyNumCopies :: Key -> AssociatedFile -> NumCopies -> Annex Bool
 checkKeyNumCopies key afile numcopies = do
 	let (desc, hasafile) = case afile of
-		AssociatedFile Nothing -> (serializeKey key, False)
-		AssociatedFile (Just af) -> (fromRawFilePath af, True)
+		AssociatedFile Nothing -> (serializeKey' key, False)
+		AssociatedFile (Just af) -> (af, True)
 	locs <- loggedLocations key
 	(untrustedlocations, otherlocations) <- trustPartition UnTrusted locs
 	(deadlocations, safelocations) <- trustPartition DeadTrusted otherlocations
@@ -589,21 +585,21 @@ checkKeyNumCopies key afile numcopies = do
 			)
 		else return True
 
-missingNote :: String -> Int -> NumCopies -> String -> String -> String
+missingNote :: RawFilePath -> Int -> NumCopies -> String -> String -> StringContainingQuotedPath
 missingNote file 0 _ [] dead = 
-		"** No known copies exist of " ++ file ++ honorDead dead
+		"** No known copies exist of " <> QuotedPath file <> UnquotedString (honorDead dead)
 missingNote file 0 _ untrusted dead =
-		"Only these untrusted locations may have copies of " ++ file ++
-		"\n" ++ untrusted ++
-		"Back it up to trusted locations with git-annex copy." ++ honorDead dead
+		"Only these untrusted locations may have copies of " <> QuotedPath file <>
+		"\n" <> UnquotedString untrusted <>
+		"Back it up to trusted locations with git-annex copy." <> UnquotedString (honorDead dead)
 missingNote file present needed [] _ =
-		"Only " ++ show present ++ " of " ++ show (fromNumCopies needed) ++ 
-		" trustworthy copies exist of " ++ file ++
+		"Only " <> UnquotedString (show present) <> " of " <> UnquotedString (show (fromNumCopies needed)) <>
+		" trustworthy copies exist of " <> QuotedPath file <>
 		"\nBack it up with git-annex copy."
 missingNote file present needed untrusted dead = 
-		missingNote file present needed [] dead ++
-		"\nThe following untrusted locations may also have copies: " ++
-		"\n" ++ untrusted
+		missingNote file present needed [] dead <>
+		"\nThe following untrusted locations may also have copies: " <>
+		"\n" <> UnquotedString untrusted
 	
 honorDead :: String -> String
 honorDead dead
