@@ -43,7 +43,6 @@ import Logs.Transfer
 import Types.Key
 import Types.TrustLevel
 import Types.FileMatcher
-import Types.ActionItem
 import qualified Limit
 import Messages.JSON (DualDisp(..), ObjectMap(..))
 import Annex.BloomFilter
@@ -183,8 +182,12 @@ itemInfo o (si, p) = ifM (isdir (toRawFilePath p))
 
 noInfo :: String -> SeekInput -> String -> Annex ()
 noInfo s si msg = do
-	showStart "info" (encodeBS s) si
-	showNote msg
+	-- The string may not really be a file, but use ActionItemTreeFile,
+	-- rather than ActionItemOther to avoid breaking back-compat of
+	-- json output.
+	let ai = ActionItemTreeFile (toRawFilePath s)
+	showStartMessage (StartMessage "info" ai si)
+	showNote (UnquotedString msg)
 	showEndFail
 	Annex.incError
 
@@ -455,15 +458,16 @@ transfer_list = stat desc $ nojson $ lift $ do
 	uuidmap <- Remote.remoteMap id
 	ts <- getTransfers
 	maybeShowJSON $ JSONChunk [(desc, V.fromList $ map (uncurry jsonify) ts)]
+	qp <- coreQuotePath <$> Annex.getGitConfig
 	return $ if null ts
 		then "none"
 		else multiLine $
-			map (uncurry $ line uuidmap) $ sort ts
+			map (uncurry $ line qp uuidmap) $ sort ts
   where
 	desc = "transfers in progress"
-	line uuidmap t i = unwords
+	line qp uuidmap t i = unwords
 		[ fromRawFilePath (formatDirection (transferDirection t)) ++ "ing"
-		, fromRawFilePath $ actionItemDesc $ mkActionItem
+		, fromRawFilePath $ quote qp $ actionItemDesc $ mkActionItem
 			(transferKey t, associatedFile i)
 		, if transferDirection t == Upload then "to" else "from"
 		, maybe (fromUUID $ transferUUID t) Remote.name $
