@@ -6,6 +6,7 @@
  -}
 
 {-# LANGUAGE RankNTypes #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 module Remote.Adb (remote) where
 
@@ -15,6 +16,7 @@ import Types.Creds
 import Types.Export
 import Types.Import
 import qualified Git
+import Config
 import Config.Cost
 import Remote.Helper.Special
 import Remote.Helper.ExportImport
@@ -46,6 +48,8 @@ remote = specialRemoteType $ RemoteType
 			(FieldDesc "sometimes needed to specify which Android device to use")
 		, yesNoParser ignorefinderrorField (Just False)
 			(FieldDesc "ignore adb find errors")
+		, yesNoParser oldandroidField (Just False)
+			(FieldDesc "support old versions of android (slower)")
 		]
 	, setup = adbSetup
 	, exportSupported = exportIsSupported
@@ -62,15 +66,18 @@ androidserialField = Accepted "androidserial"
 ignorefinderrorField :: RemoteConfigField
 ignorefinderrorField = Accepted "ignorefinderror"
 
+oldandroidField :: RemoteConfigField
+oldandroidField = Accepted "oldandroid"
 
 gen :: Git.Repo -> UUID -> RemoteConfig -> RemoteGitConfig -> RemoteStateHandle -> Annex (Maybe Remote)
 gen r u rc gc rs = do
 	c <- parsedRemoteConfig remote rc
+	-- adb operates over USB or wifi, so is not as cheap
+	-- as local, but not too expensive
+	cst <- remoteCost gc c semiExpensiveRemoteCost
 	let this = Remote
 		{ uuid = u
-		-- adb operates over USB or wifi, so is not as cheap
-		-- as local, but not too expensive
-		, cost = semiExpensiveRemoteCost
+		, cost = cst
 		, name = Git.repoDescribe r
 		, storeKey = storeKeyDummy
 		, retrieveKeyFile = retrieveKeyFileDummy
@@ -305,26 +312,44 @@ listImportableContentsM serial adir c = adbfind >>= \case
 		ImportableContents (mapMaybe mk ls) []
 	Nothing -> giveup "adb find failed"
   where
-	adbfind = adbShell' serial
+	adbfind = adbShell' serial findparams
+		(\s -> if oldandroid
+			then s ++ ignorefinderrorsh
+			else concat
+				[ "set -o pipefail; "
+				, s
+				, "| xargs -0 stat -c " ++ shellEscape statformat ++ " --"
+				, ignorefinderrorsh
+				]
+		)
+	
+	findparams =
 		[ Param "find"
 		-- trailing slash is needed, or android's find command
 		-- won't recurse into the directory
 		, File $ fromAndroidPath adir ++ "/"
 		, Param "-type", Param "f"
-		, Param "-printf", Param "%p\\0"
-		] 
-		(\s -> concat
-			[ "set -o pipefail; "
-			, s
-			, "| xargs -0 stat -c " ++ shellEscape statformat ++ " --"
-			, if ignorefinderror then " || true" else ""
-			]
-		)
+		] ++ if oldandroid
+			then 
+				[ Param "-exec"
+				, Param "stat"
+				, Param "-c"
+				, Param statformat
+				, Param "{}"
+				, Param ";"
+				]
+			else 
+				[ Param "-printf"
+				, Param "%p\\0"
+				]
 
 	ignorefinderror = fromMaybe False (getRemoteConfigValue ignorefinderrorField c)
+	oldandroid = fromMaybe False (getRemoteConfigValue oldandroidField c)
 
 	statformat = adbStatFormat ++ "\t%n"
 
+	ignorefinderrorsh = if ignorefinderror then " || true" else ""
+
 	mk ('S':'T':'\t':l) =
 		let (stat, fn) = separate (== '\t') l
 		    sz = fromMaybe 0 (readish (takeWhile (/= ' ') stat))
@@ -338,8 +363,8 @@ listImportableContentsM serial adir c = adbfind >>= \case
 -- connection is resonably fast, it's probably as good as
 -- git's handling of similar situations with files being modified while
 -- it's updating the working tree for a merge.
-retrieveExportWithContentIdentifierM :: AndroidSerial -> AndroidPath -> ExportLocation -> ContentIdentifier -> FilePath -> Either Key (Annex Key) -> MeterUpdate -> Annex (Key, Verification)
-retrieveExportWithContentIdentifierM serial adir loc cid dest gk _p = do
+retrieveExportWithContentIdentifierM :: AndroidSerial -> AndroidPath -> ExportLocation -> [ContentIdentifier] -> FilePath -> Either Key (Annex Key) -> MeterUpdate -> Annex (Key, Verification)
+retrieveExportWithContentIdentifierM serial adir loc cids dest gk _p = do
 	case gk of
 		Right mkkey -> do
 			go
@@ -352,9 +377,10 @@ retrieveExportWithContentIdentifierM serial adir loc cid dest gk _p = do
   where
 	go = do
 		retrieve' serial src dest
-		currcid <- getExportContentIdentifier serial adir loc
-		when (currcid /= Right (Just cid)) $
-			giveup "the file on the android device has changed"
+		getExportContentIdentifier serial adir loc >>= \case
+			Right (Just currcid)
+				| any (currcid ==) cids -> return ()
+			_ -> giveup "the file on the android device has changed"
 	src = androidExportLocation adir loc
 
 storeExportWithContentIdentifierM :: AndroidSerial -> AndroidPath -> FilePath -> Key -> ExportLocation -> [ContentIdentifier] -> MeterUpdate -> Annex ContentIdentifier
