@@ -11,8 +11,10 @@ import Annex.Common
 import qualified Git
 import Types.Availability
 import qualified Types.Remote as Remote
+import qualified Utility.RawFilePath as R
 
 import Data.Time.Clock.POSIX
+import System.PosixCompat.Files (modificationTime)
 
 repoCheap :: Git.Repo -> Bool
 repoCheap = not . Git.repoIsUrl
@@ -20,7 +22,7 @@ repoCheap = not . Git.repoIsUrl
 localpathCalc :: Git.Repo -> Maybe FilePath
 localpathCalc r
 	| availabilityCalc r == GloballyAvailable = Nothing
-	| otherwise = Just $ Git.repoPath r
+	| otherwise = Just $ fromRawFilePath $ Git.repoPath r
 
 availabilityCalc :: Git.Repo -> Availability
 availabilityCalc r
@@ -36,8 +38,8 @@ guardUsable r fallback a
 
 gitRepoInfo :: Remote -> Annex [(String, String)]
 gitRepoInfo r = do
-	d <- fromRepo Git.localGitDir
-	mtimes <- liftIO $ mapM (modificationTime <$$> getFileStatus)
+	d <- fromRawFilePath <$> fromRepo Git.localGitDir
+	mtimes <- liftIO $ mapM (\p -> modificationTime <$> R.getFileStatus (toRawFilePath p))
 		=<< dirContentsRecursive (d </> "refs" </> "remotes" </> Remote.name r)
 	let lastsynctime = case mtimes of
 		[] -> "never"
